@@ -127,6 +127,9 @@ def get_and_parse_backend(infile, settings, temp_files, DEBUG):
         backend_args += ['-emscripten-global-base=0']
       elif settings['GLOBAL_BASE'] >= 0:
         backend_args += ['-emscripten-global-base=%d' % settings['GLOBAL_BASE']]
+      if settings['SIDE_MODULE']:
+        backend_args += ['-emscripten-side-module']
+      backend_args += ['-emscripten-stack-size=%d' % settings['TOTAL_STACK']]
       backend_args += ['-O' + str(settings['OPT_LEVEL'])]
       if settings['DISABLE_EXCEPTION_CATCHING'] != 1:
         backend_args += ['-enable-emscripten-cxx-exceptions']
@@ -376,6 +379,9 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
            (('function ' + requested.encode('utf-8')) not in pre): # could be a js library func
           logging.warning('function requested to be exported, but not implemented: "%s"', requested)
 
+    if settings['BINARYEN'] and settings['SIDE_MODULE']:
+      assert len(metadata['asmConsts']) == 0, 'EM_ASM is not yet supported in shared wasm module (it cannot be stored in the wasm itself, need some solution)'
+
     asm_consts = [0]*len(metadata['asmConsts'])
     all_sigs = []
     for k, v in metadata['asmConsts'].iteritems():
@@ -454,6 +460,7 @@ function _emscripten_asm_const_%s(%s) {
     in_table = set()
 
     def make_table(sig, raw):
+      if '[]' in raw: return ('', '') # empty table
       params = make_params(sig)
       coerced_params = make_coerced_params(sig)
       coercions = make_coercions(sig)
@@ -532,8 +539,8 @@ function _emscripten_asm_const_%s(%s) {
         clean_item = item.replace("asm['", '').replace("']", '')
         # when emulating function pointers, we don't need wrappers
         # but if relocating, then we also have the copies in-module, and do
-        # also if binaryen, as wasm requires wrappers for the function table
-        if clean_item not in implemented_functions and not (settings['EMULATED_FUNCTION_POINTERS'] and not settings['RELOCATABLE'] and not settings['BINARYEN']):
+        # in wasm we never need wrappers though
+        if clean_item not in implemented_functions and not (settings['EMULATED_FUNCTION_POINTERS'] and not settings['RELOCATABLE']) and not settings['BINARYEN']:
           # this is imported into asm, we must wrap it
           call_ident = clean_item
           if call_ident in metadata['redirects']: call_ident = metadata['redirects'][call_ident]
@@ -545,6 +552,7 @@ function _emscripten_asm_const_%s(%s) {
             code = 'return ' + shared.JS.make_coercion(code, sig[0], settings)
           code += ';'
           Counter.pre.append(make_func(clean_item + '__wrapper', code, params, coercions))
+          assert not sig == 'X', 'must know the signature in order to create a wrapper for "%s" (TODO for shared wasm modules)' % item
           return clean_item + '__wrapper'
         return item if not newline else (item + '\n')
       if settings['ASSERTIONS'] >= 2:
@@ -675,7 +683,9 @@ function _emscripten_asm_const_%s(%s) {
         basic_funcs += ['nullFunc_' + sig]
         asm_setup += '\nfunction nullFunc_' + sig + '(x) { ' + get_function_pointer_error(sig) + 'abort(x) }\n'
 
-    basic_vars = ['STACKTOP', 'STACK_MAX', 'DYNAMICTOP_PTR', 'tempDoublePtr', 'ABORT']
+    basic_vars = ['DYNAMICTOP_PTR', 'tempDoublePtr', 'ABORT']
+    if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+      basic_vars += ['STACKTOP', 'STACK_MAX']
     basic_float_vars = []
 
     if metadata.get('preciseI64MathUsed'):
@@ -685,11 +695,16 @@ function _emscripten_asm_const_%s(%s) {
         basic_vars += ['cttz_i8']
 
     if settings['RELOCATABLE']:
-      basic_vars += ['gb', 'fb']
+      if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+        basic_vars += ['gb', 'fb']
+      else:
+        basic_vars += ['memoryBase', 'tableBase'] # wasm side modules have a specific convention for these
       if not settings['SIDE_MODULE']:
         asm_setup += 'var gb = Runtime.GLOBAL_BASE, fb = 0;\n'
 
-    asm_runtime_funcs = ['stackAlloc', 'stackSave', 'stackRestore', 'establishStackSpace', 'setThrew']
+    asm_runtime_funcs = []
+    if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
+      asm_runtime_funcs += ['stackAlloc', 'stackSave', 'stackRestore', 'establishStackSpace', 'setThrew']
     if not settings['RELOCATABLE']:
       asm_runtime_funcs += ['setTempRet0', 'getTempRet0']
     else:
@@ -697,7 +712,16 @@ function _emscripten_asm_const_%s(%s) {
       asm_setup += 'var setTempRet0 = Runtime.setTempRet0, getTempRet0 = Runtime.getTempRet0;\n'
 
     if settings['BINARYEN']:
-      asm_setup += "\nModule['wasmTableSize'] = %d;\n" % sum(map(lambda table: table.count(',') + 1, last_forwarded_json['Functions']['tables'].values()))
+      def table_size(table):
+        table_contents = table[table.index('[') + 1: table.index(']')]
+        if len(table_contents) == 0: # empty table
+          return 0
+        return table_contents.count(',') + 1
+
+      table_total_size = sum(map(table_size, last_forwarded_json['Functions']['tables'].values()))
+      asm_setup += "\nModule['wasmTableSize'] = %d;\n" % table_total_size
+      if not settings['EMULATED_FUNCTION_POINTERS']:
+        asm_setup += "\nModule['wasmMaxTableSize'] = %d;\n" % table_total_size
 
     # See if we need ASYNCIFY functions
     # We might not need them even if ASYNCIFY is enabled
@@ -764,16 +788,23 @@ function jsCall_%s_%s(%s) {
         table_access = 'FUNCTION_TABLE_' + sig
         if settings['SIDE_MODULE']:
           table_access = 'parentModule["' + table_access + '"]' # side module tables were merged into the parent, we need to access the global one
+        if settings['BINARYEN']:
+          # wasm uses a Table, which means we have function pointer emulation capabilities all the time, at no cost. just call the table
+          table_access = "Module['wasmTable']"
+          table_read = table_access + '.get(x)'
+        else:
+          table_read = table_access + '[x]'
         prelude = '''
   if (x < 0 || x >= %s.length) { Module.printErr("Function table mask error (out of range)"); %s ; abort(x) }''' % (table_access, get_function_pointer_error(sig))
         asm_setup += '''
 function ftCall_%s(%s) {%s
-  return %s[x](%s);
+  return %s(%s);
 }
-''' % (sig, ', '.join(full_args), prelude, table_access, ', '.join(args))
-        basic_funcs.append('ftCall_%s' % sig)
+''' % (sig, ', '.join(full_args), prelude, table_read, ', '.join(args))
+        if not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
+          basic_funcs.append('ftCall_%s' % sig)
 
-        if settings.get('RELOCATABLE'):
+        if settings.get('RELOCATABLE') and not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
           params = ','.join(['ptr'] + ['p%d' % p for p in range(len(sig)-1)])
           coerced_params = ','.join([shared.JS.make_coercion('ptr', 'i', settings)] + [shared.JS.make_coercion('p%d', unfloat(sig[p+1]), settings) % p for p in range(len(sig)-1)])
           coercions = ';'.join(['ptr = ptr | 0'] + ['p%d = %s' % (p, shared.JS.make_coercion('p%d' % p, unfloat(sig[p+1]), settings)) for p in range(len(sig)-1)]) + ';'
@@ -799,6 +830,13 @@ function ftCall_%s(%s) {%s
     exports = []
     for export in all_exported:
       exports.append(quote(export) + ": " + export)
+    if settings['BINARYEN'] and settings['SIDE_MODULE']:
+      # named globals in side wasm modules are exported globals from asm/wasm
+      for k, v in metadata['namedGlobals'].iteritems():
+        exports.append(quote('_' + str(k)) + ': ' + str(v))
+      # aliases become additional exports
+      for k, v in metadata['aliases'].iteritems():
+        exports.append(quote(str(k)) + ': ' + str(v))
     exports = '{ ' + ', '.join(exports) + ' }'
     # calculate globals
     try:
@@ -871,6 +909,9 @@ function ftCall_%s(%s) {%s
       asm_global_funcs += ''.join(['  var Atomics_' + ty + '=global' + access_quote('Atomics') + access_quote(ty) + ';\n' for ty in ['load', 'store', 'exchange', 'compareExchange', 'add', 'sub', 'and', 'or', 'xor']])
     asm_global_vars = ''.join(['  var ' + g + '=env' + access_quote(g) + '|0;\n' for g in basic_vars + global_vars])
 
+    if settings['BINARYEN'] and settings['SIDE_MODULE']:
+      asm_global_vars += '\n  var STACKTOP = 0, STACK_MAX = 0;\n' # wasm side modules internally define their stack, these are set at module startup time
+
     # sent data
     the_global = '{ ' + ', '.join(['"' + math_fix(s) + '": ' + s for s in fundamentals]) + ' }'
     sending = '{ ' + ', '.join(['"' + math_fix(s) + '": ' + s for s in basic_funcs + global_funcs + basic_vars + basic_float_vars + global_vars]) + ' }'
@@ -884,7 +925,7 @@ assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. w
 assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
 return real_''' + s + '''.apply(null, arguments);
 };
-''' for s in exported_implemented_functions if s not in ['_memcpy', '_memset', 'runPostSets', '_emscripten_replace_memory']])
+''' for s in exported_implemented_functions if s not in ['_memcpy', '_memset', 'runPostSets', '_emscripten_replace_memory', '__start_module']])
 
     if not settings['SWAPPABLE_ASM_MODULE']:
       receiving += ';\n'.join(['var ' + s + ' = Module["' + s + '"] = asm["' + s + '"]' for s in exported_implemented_functions + function_tables])
@@ -892,7 +933,7 @@ return real_''' + s + '''.apply(null, arguments);
       receiving += 'Module["asm"] = asm;\n' + ';\n'.join(['var ' + s + ' = Module["' + s + '"] = function() { return Module["asm"]["' + s + '"].apply(null, arguments) }' for s in exported_implemented_functions + function_tables])
     receiving += ';\n'
 
-    if settings['EXPORT_FUNCTION_TABLES']:
+    if settings['EXPORT_FUNCTION_TABLES'] and not settings['BINARYEN']:
       for table in last_forwarded_json['Functions']['tables'].values():
         tableName = table.split()[1]
         table = table.replace('var ' + tableName, 'var ' + tableName + ' = Module["' + tableName + '"]')
@@ -904,10 +945,11 @@ return real_''' + s + '''.apply(null, arguments);
     if settings.get('EMULATED_FUNCTION_POINTERS'):
       asm_setup += '\n' + '\n'.join(function_tables_impls) + '\n'
       receiving += '\n' + function_tables_defs.replace('// EMSCRIPTEN_END_FUNCS\n', '') + '\n' + ''.join(['Module["dynCall_%s"] = dynCall_%s\n' % (sig, sig) for sig in last_forwarded_json['Functions']['tables']])
-      for sig in last_forwarded_json['Functions']['tables'].keys():
-        name = 'FUNCTION_TABLE_' + sig
-        fullname = name if not settings['SIDE_MODULE'] else ('SIDE_' + name)
-        receiving += 'Module["' + name + '"] = ' + fullname + ';\n'
+      if not settings['BINARYEN']:
+        for sig in last_forwarded_json['Functions']['tables'].keys():
+          name = 'FUNCTION_TABLE_' + sig
+          fullname = name if not settings['SIDE_MODULE'] else ('SIDE_' + name)
+          receiving += 'Module["' + name + '"] = ' + fullname + ';\n'
 
       final_function_tables = final_function_tables.replace("asm['", '').replace("']", '').replace('var SIDE_FUNCTION_TABLE_', 'var FUNCTION_TABLE_').replace('var dynCall_', '//')
 
@@ -932,11 +974,22 @@ for (var named in NAMED_GLOBALS) {
 }
 Module['NAMED_GLOBALS'] = NAMED_GLOBALS;
 ''' % ', '.join('"' + k + '": ' + str(v) for k, v in metadata['namedGlobals'].iteritems())
-
+      if settings['BINARYEN']:
+        # wasm side modules are pure wasm, and cannot create their g$..() methods, so we help them out
+        # TODO: this works if we are the main module, but if the supplying module is later, it won't, so
+        #       we'll need another solution for that. one option is to scan the module imports, if/when
+        #       wasm supports that, then the loader can do this.
+        receiving += '''
+for (var named in NAMED_GLOBALS) {
+  (function(named) {
+    Module['g$_' + named] = function() { return Module['_' + named] };
+  })(named);
+}
+'''
       receiving += ''.join(["Module['%s'] = Module['%s']\n" % (k, v) for k, v in metadata['aliases'].iteritems()])
 
     if settings['USE_PTHREADS']:
-      shared_array_buffer = "if (typeof SharedArrayBuffer !== 'undefined') Module.asmGlobalArg['Atomics'] = Atomics;"
+      shared_array_buffer = "Module.asmGlobalArg['Atomics'] = Atomics;"
     else:
       shared_array_buffer = ''
 
@@ -1352,7 +1405,8 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
       shutil.copyfile(temp_s, os.path.join(shared.CANONICAL_TEMP_DIR, 'emcc-llvm-backend-output.s'))
 
     assert shared.Settings.BINARYEN_ROOT, 'need BINARYEN_ROOT config set so we can use Binaryen s2wasm on the backend output'
-    wasm = outfile.name[:-3] + '.wast'
+    basename = outfile.name[:-3]
+    wast = basename + '.wast'
     s2wasm_args = [os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 's2wasm'), temp_s]
     s2wasm_args += ['--emscripten-glue']
     s2wasm_args += ['--global-base=%d' % shared.Settings.GLOBAL_BASE]
@@ -1365,13 +1419,18 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
       logging.debug('emscript: binaryen s2wasm: ' + ' '.join(s2wasm_args))
       t = time.time()
       #s2wasm_args += ['--debug']
-    shared.check_call(s2wasm_args, stdout=open(wasm, 'w'))
+    shared.check_call(s2wasm_args, stdout=open(wast, 'w'))
+    # Also convert wasm text to binary
+    wasm_as_args = [os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-as'),
+                    wast, '-o', basename + '.wasm']
+    logging.debug('  emscript: binaryen wasm-as: ' + ' '.join(wasm_as_args))
+    shared.check_call(wasm_as_args)
 
   if DEBUG:
     logging.debug('  emscript: binaryen s2wasm took %s seconds' % (time.time() - t))
     t = time.time()
     import shutil
-    shutil.copyfile(wasm, os.path.join(shared.CANONICAL_TEMP_DIR, 'emcc-s2wasm-output.wast'))
+    shutil.copyfile(wast, os.path.join(shared.CANONICAL_TEMP_DIR, 'emcc-s2wasm-output.wast'))
 
   # js compiler
 
@@ -1379,7 +1438,7 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
 
   # Integrate info from backend
 
-  output = open(wasm).read()
+  output = open(wast).read()
   parts = output.split('\n;; METADATA:')
   assert len(parts) == 2
   metadata_raw = parts[1]
@@ -1415,7 +1474,7 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
   metadata['initializers'] = [asmjs_mangle(i) for i in metadata['initializers']]
 
   # TODO: emit it from s2wasm; for now, we parse it right here
-  for line in open(wasm).readlines():
+  for line in open(wast).readlines():
     if line.startswith('  (import '):
       parts = line.split()
       # Don't include Invoke wrapper names (for asm.js-style exception handling)
@@ -1582,7 +1641,7 @@ return ASM_CONSTS[code](%s);
 
   # Asm.js-style exception handling: invoke wrapper generation
   invoke_wrappers = ''
-  with open(wasm) as f:
+  with open(wast) as f:
     for line in f:
       if line.startswith('  (import '):
         parts = line.split()
@@ -1605,7 +1664,7 @@ assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. w
 assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
 return real_''' + asmjs_mangle(s) + '''.apply(null, arguments);
 };
-''' for s in exported_implemented_functions if s not in ['_memcpy', '_memset', 'runPostSets', '_emscripten_replace_memory']])
+''' for s in exported_implemented_functions if s not in ['_memcpy', '_memset', 'runPostSets', '_emscripten_replace_memory', '__start_module']])
 
   if not settings['SWAPPABLE_ASM_MODULE']:
     receiving += ';\n'.join(['var ' + asmjs_mangle(s) + ' = Module["' + asmjs_mangle(s) + '"] = asm["' + s + '"]' for s in exported_implemented_functions])
