@@ -736,35 +736,6 @@ var LibraryBrowser = {
     }
   },
 
-#if ASYNCIFY
-  emscripten_wget__deps: ['emscripten_async_resume', '$PATH'],
-  emscripten_wget: function(url, file) {
-    var _url = Pointer_stringify(url);
-    var _file = Pointer_stringify(file);
-    _file = PATH.resolve(FS.cwd(), _file);
-    asm.setAsync();
-    Module['noExitRuntime'] = true;
-    var destinationDirectory = PATH.dirname(_file);
-    FS.createPreloadedFile(
-      destinationDirectory,
-      PATH.basename(_file),
-      _url, true, true,
-      _emscripten_async_resume,
-      _emscripten_async_resume,
-      undefined, // dontCreateFile
-      undefined, // canOwn
-      function() { // preFinish
-        // if the destination directory does not yet exist, create it
-        FS.mkdirTree(destinationDirectory);
-      }
-    );
-  },
-#else
-  emscripten_wget: function(url, file) {
-    throw 'Please compile your program with -s ASYNCIFY=1 in order to use asynchronous operations like emscripten_wget';
-  },
-#endif
-
   emscripten_async_wget__deps: ['$PATH'],
   emscripten_async_wget: function(url, file, onload, onerror) {
     Module['noExitRuntime'] = true;
@@ -775,7 +746,7 @@ var LibraryBrowser = {
     function doCallback(callback) {
       if (callback) {
         var stack = Runtime.stackSave();
-        Runtime.dynCall('vi', callback, [allocate(intArrayFromString(_file), 'i8', ALLOC_STACK)]);
+        Module['dynCall_vi'](callback, allocate(intArrayFromString(_file), 'i8', ALLOC_STACK));
         Runtime.stackRestore(stack);
       }
     }
@@ -803,39 +774,14 @@ var LibraryBrowser = {
     );
   },
 
-#if EMTERPRETIFY_ASYNC
-  emscripten_wget_data__deps: ['$EmterpreterAsync'],
-  emscripten_wget_data: function(url, pbuffer, pnum, perror) {
-    EmterpreterAsync.handle(function(resume) {
-      Browser.asyncLoad(Pointer_stringify(url), function(byteArray) {
-        resume(function() {
-          // can only allocate the buffer after the resume, not during an asyncing
-          var buffer = _malloc(byteArray.length); // must be freed by caller!
-          HEAPU8.set(byteArray, buffer);
-          {{{ makeSetValueAsm('pbuffer', 0, 'buffer', 'i32') }}};
-          {{{ makeSetValueAsm('pnum',  0, 'byteArray.length', 'i32') }}};
-          {{{ makeSetValueAsm('perror',  0, '0', 'i32') }}};
-        });
-      }, function() {
-        {{{ makeSetValueAsm('perror',  0, '1', 'i32') }}};
-        resume();
-      }, true /* no need for run dependency, this is async but will not do any prepare etc. step */ );
-    });
-  },
-#else
-  emscripten_wget_data: function(url, file) {
-    throw 'Please compile your program with -s EMTERPRETER_ASYNC=1 in order to use asynchronous operations like emscripten_wget_data';
-  },
-#endif
-
   emscripten_async_wget_data: function(url, arg, onload, onerror) {
     Browser.asyncLoad(Pointer_stringify(url), function(byteArray) {
       var buffer = _malloc(byteArray.length);
       HEAPU8.set(byteArray, buffer);
-      Runtime.dynCall('viii', onload, [arg, buffer, byteArray.length]);
+      Module['dynCall_viii'](onload, arg, buffer, byteArray.length);
       _free(buffer);
     }, function() {
-      if (onerror) Runtime.dynCall('vi', onerror, [arg]);
+      if (onerror) Module['dynCall_vi'](onerror, arg);
     }, true /* no need for run dependency, this is async but will not do any prepare etc. step */ );
   },
 
@@ -870,11 +816,11 @@ var LibraryBrowser = {
         FS.createDataFile( _file.substr(0, index), _file.substr(index + 1), new Uint8Array(http.response), true, true, false);
         if (onload) {
           var stack = Runtime.stackSave();
-          Runtime.dynCall('viii', onload, [handle, arg, allocate(intArrayFromString(_file), 'i8', ALLOC_STACK)]);
+          Module['dynCall_viii'](onload, handle, arg, allocate(intArrayFromString(_file), 'i8', ALLOC_STACK));
           Runtime.stackRestore(stack);
         }
       } else {
-        if (onerror) Runtime.dynCall('viii', onerror, [handle, arg, http.status]);
+        if (onerror) Module['dynCall_viii'](onerror, handle, arg, http.status);
       }
 
       delete Browser.wgetRequests[handle];
@@ -882,7 +828,7 @@ var LibraryBrowser = {
 
     // ERROR
     http.onerror = function http_onerror(e) {
-      if (onerror) Runtime.dynCall('viii', onerror, [handle, arg, http.status]);
+      if (onerror) Module['dynCall_viii'](onerror, handle, arg, http.status);
       delete Browser.wgetRequests[handle];
     };
 
@@ -890,7 +836,7 @@ var LibraryBrowser = {
     http.onprogress = function http_onprogress(e) {
       if (e.lengthComputable || (e.lengthComputable === undefined && e.total != 0)) {
         var percentComplete = (e.loaded / e.total)*100;
-        if (onprogress) Runtime.dynCall('viii', onprogress, [handle, arg, percentComplete]);
+        if (onprogress) Module['dynCall_viii'](onprogress, handle, arg, percentComplete);
       }
     };
 
@@ -935,10 +881,10 @@ var LibraryBrowser = {
         var byteArray = new Uint8Array(http.response);
         var buffer = _malloc(byteArray.length);
         HEAPU8.set(byteArray, buffer);
-        if (onload) Runtime.dynCall('viiii', onload, [handle, arg, buffer, byteArray.length]);
+        if (onload) Module['dynCall_viiii'](onload, handle, arg, buffer, byteArray.length);
         if (free) _free(buffer);
       } else {
-        if (onerror) Runtime.dynCall('viiii', onerror, [handle, arg, http.status, http.statusText]);
+        if (onerror) Module['dynCall_viiii'](onerror, handle, arg, http.status, http.statusText);
       }
       delete Browser.wgetRequests[handle];
     };
@@ -946,14 +892,14 @@ var LibraryBrowser = {
     // ERROR
     http.onerror = function http_onerror(e) {
       if (onerror) {
-        Runtime.dynCall('viiii', onerror, [handle, arg, http.status, http.statusText]);
+        Module['dynCall_viiii'](onerror, handle, arg, http.status, http.statusText);
       }
       delete Browser.wgetRequests[handle];
     };
 
     // PROGRESS
     http.onprogress = function http_onprogress(e) {
-      if (onprogress) Runtime.dynCall('viiii', onprogress, [handle, arg, e.loaded, e.lengthComputable || e.lengthComputable === undefined ? e.total : 0]);
+      if (onprogress) Module['dynCall_viiii'](onprogress, handle, arg, e.loaded, e.lengthComputable || e.lengthComputable === undefined ? e.total : 0);
     };
 
     // ABORT
@@ -999,10 +945,10 @@ var LibraryBrowser = {
       PATH.basename(_file),
       new Uint8Array(data.object.contents), true, true,
       function() {
-        if (onload) Runtime.dynCall('vi', onload, [file]);
+        if (onload) Module['dynCall_vi'](onload, file);
       },
       function() {
-        if (onerror) Runtime.dynCall('vi', onerror, [file]);
+        if (onerror) Module['dynCall_vi'](onerror, file);
       },
       true // don'tCreateFile - it's already there
     );
@@ -1024,10 +970,10 @@ var LibraryBrowser = {
       {{{ makeHEAPView('U8', 'data', 'data + size') }}},
       true, true,
       function() {
-        if (onload) Runtime.dynCall('vii', onload, [arg, cname]);
+        if (onload) Module['dynCall_vii'](onload, arg, cname);
       },
       function() {
-        if (onerror) Runtime.dynCall('vi', onerror, [arg]);
+        if (onerror) Module['dynCall_vi'](onerror, arg);
       },
       true // don'tCreateFile - it's already there
     );
@@ -1130,13 +1076,12 @@ var LibraryBrowser = {
 
     var browserIterationFunc;
     if (typeof arg !== 'undefined') {
-      var argArray = [arg];
       browserIterationFunc = function() {
-        Runtime.dynCall('vi', func, argArray);
+        Module['dynCall_vi'](func, arg);
       };
     } else {
       browserIterationFunc = function() {
-        Runtime.dynCall('v', func);
+        Module['dynCall_v'](func);
       };
     }
 
@@ -1251,14 +1196,14 @@ var LibraryBrowser = {
 
   _emscripten_push_main_loop_blocker: function(func, arg, name) {
     Browser.mainLoop.queue.push({ func: function() {
-      Runtime.dynCall('vi', func, [arg]);
+      Module['dynCall_vi'](func, arg);
     }, name: Pointer_stringify(name), counted: true });
     Browser.mainLoop.updateStatus();
   },
 
   _emscripten_push_uncounted_main_loop_blocker: function(func, arg, name) {
     Browser.mainLoop.queue.push({ func: function() {
-      Runtime.dynCall('vi', func, [arg]);
+      Module['dynCall_vi'](func, arg);
     }, name: Pointer_stringify(name), counted: false });
     Browser.mainLoop.updateStatus();
   },
